@@ -233,10 +233,12 @@ class TheaterOptimizer(Enum):
     off = b"0"
     on = b"1"
 
+
 class LampPowerModes(Enum):
     normal = b"0"
     high = b"1"
 
+
 class HdrData(Enum):
     sdr = b"0"
     hdr = b"1"
@@ -245,6 +247,13 @@ class HdrData(Enum):
     hdr10_plus = b"4"
     none = b"F"
 
+
+class AspectRatioModes(Enum):
+    zoom = b"2"
+    auto = b"3"
+    native = b"4"
+
+
 class Commands(Enum):
 
     # these use ! unless otherwise indicated
@@ -285,6 +294,9 @@ class Commands(Enum):
     # Color modes
     color_mode = b"ISHS", ColorSpaceModes, ACKs.hdmi_ack
 
+    # Aspect ratio
+    aspect_ratio = b"ISAS", AspectRatioModes, ACKs.hdmi_ack
+
     # input_level like 0-255
     input_level = b"ISIL", InputLevel, ACKs.hdmi_ack
 
@@ -298,7 +310,7 @@ class Commands(Enum):
     graphic_mode = b"PMGM", GraphicModeModes, ACKs.picture_ack
 
     # mask commands
-    mask = b"ISMA", MaskModes
+    mask = b"ISMA", MaskModes, ACKs.hdmi_ack
 
     # laser power commands
     laser_power = b"PMLP", LaserPowerModes, ACKs.picture_ack
