@@ -25,7 +25,9 @@ from jvc_projector.commands import (
     HdrProcessing,
     TheaterOptimizer,
     HdrData,
-    LampPowerModes
+    LampPowerModes,
+    LaserPowerModes,
+    AspectRatioModes
 )
 
 
@@ -155,10 +157,13 @@ class JVCProjector:
             "A2B1": "NX9",
             "A2B2": "NX7",
             "A2B3": "NX5",
+            "B2A1": "NX9",
+            "B2A2": "NX7",
+            "B2A3": "NX5",
             "B5B1": "NP5",
             "XHP1": "X5000",
             "XHP2": "XC6890",
-            "XHP3": "X7000||X9000"
+            "XHP3": "X7000||X9000",
         }
         model_res = self._replace_headers(res).decode()
         self.logger.debug(model_res)
@@ -252,11 +257,17 @@ class JVCProjector:
         ack: bytes,
         command_type: bytes = b"!",
     ) -> tuple[Union[str, bytes], bool]:
-        retry_count = 0
+
+        # ensure this doesnt run with dead client
         if self.client is None:
-            self.logger.debug("Forming connection")
+            self.logger.debug("Client is none. Reforming connection")
             self.reconnect()
 
+        # max retries
+        retry_count = 0
+        # use this to store error if retry counte exceeded
+        error = ""
+
         while retry_count < 5:
             self.logger.debug("do_command sending command: %s", command)
             # send the command
@@ -265,7 +276,6 @@ class JVCProjector:
             except ConnectionError as err:
                 # reaching this means the writer was closed somewhere
                 self.logger.error(err)
-                # self.logger.debug("Restarting connection")
                 # restart the connection
                 self.client.close()
                 self.reconnect()
@@ -286,21 +296,17 @@ class JVCProjector:
             try:
                 # most commands timeout when PJ is off
                 received_msg = self.client.recv(len(ack_value))
-            except socket.timeout:
-                self.logger.error(
-                    # TODO: this may be happening because something else read the response?
-                    "Connection timed out. Command %s may not be allowed to run at this time or something else is running already.",
-                    command,
-                )
-                self.logger.debug("restarting connection")
+            except socket.timeout as err:
+                error = f"Timed out. Command {command} may grayed out or cmd is running already."
+                self.logger.debug(err)
                 self.client.close()
                 self.reconnect()
                 retry_count += 1
-                return
+                continue
 
-            except ConnectionRefusedError:
-                self.logger.error("Connection Refused when getting msg")
-                self.logger.debug("restarting connection")
+            except ConnectionRefusedError as err:
+                error = "Connection Refused when getting msg"
+                self.logger.debug(err)
                 self.client.close()
                 self.reconnect()
                 retry_count += 1
@@ -308,41 +314,50 @@ class JVCProjector:
 
             self.logger.debug("received msg from PJ: %s", received_msg)
 
-            # This is unlikely to happen unless we read blank response
-            if received_msg == b"":
+            msg = self._check_received_msg(received_msg, ack_value, command_type)
+            if msg == b"":
                 self.logger.error("Got a blank msg. Restarting connection")
                 self.client.close()
                 self.reconnect()
                 retry_count += 1
                 continue
 
-            # get the ack for operation
-            if received_msg == ack_value and command_type == Header.operation.value:
-                return received_msg, True
-
-            # if we got what we expect and this is a reference,
-            # receive the data we requested
-            if received_msg == ack_value and command_type == Header.reference.value:
-                message = self.client.recv(1000)
-                self.logger.debug("received message from PJ: %s", message)
-
-                return message, True
-
-            # Because this now reuses a connection, reaching this stage means catastrophic failure, or HA running as usual :)
-            self.logger.error(
-                "Received ack did not match expected ack: %s != %s",
-                received_msg,
-                ack_value,
-            )
-            # Try to restart connection, if we got here somethihng is out of sync
-            self.client.close()
-            self.reconnect()
-            retry_count += 1
-            continue
+            # if all fine, return the value
+            return msg, True
 
         self.logger.error("retry count for running commands exceeded")
+        self.logger.error(error)
+
         return "retry count exceeded", None
 
+    def _check_received_msg(
+        self, received_msg: bytes, ack_value: bytes, command_type: bytes
+    ) -> bytes:
+        # This is unlikely to happen unless we read blank response
+        if received_msg == b"":
+            return received_msg
+
+        # get the ack for operation
+        if received_msg == ack_value and command_type == Header.operation.value:
+            return received_msg
+
+        # if we got what we expect and this is a reference,
+        # receive the data we requested
+        if received_msg == ack_value and command_type == Header.reference.value:
+            message = self.client.recv(1000)
+            self.logger.debug("received message from PJ: %s", message)
+
+            return message
+
+        self.logger.error(
+            "Received ack: %s != expected ack: %s",
+            received_msg,
+            ack_value,
+        )
+
+        # return blank will force it to retry
+        return b""
+
     def _construct_command(
         self, raw_command: str, command_type: bytes
     ) -> tuple[bytes, ACKs]:
@@ -536,6 +551,13 @@ class JVCProjector:
         state, _ = self._do_reference_op("lamp_power", ACKs.picture_ack)
         return LampPowerModes(state.replace(ACKs.picture_ack.value, b"")).name
 
+    def get_laser_power(self) -> str:
+        """
+        Get the current laser power NZ only
+        """
+        state, _ = self._do_reference_op("laser_power", ACKs.picture_ack)
+        return LaserPowerModes(state.replace(ACKs.picture_ack.value, b"")).name
+
     def get_theater_optimizer_state(self) -> str:
         """
         If theater optimizer is on/off Will fail if not in HDR mode!
@@ -543,35 +565,21 @@ class JVCProjector:
         state, _ = self._do_reference_op("theater_optimizer", ACKs.picture_ack)
         return TheaterOptimizer(state.replace(ACKs.picture_ack.value, b"")).name
 
+    def get_aspect_ratio(self) -> str:
+        """
+        Return aspect ratio
+        """
+        state, _ = self._do_reference_op("aspect_ratio", ACKs.hdmi_ack)
+        return AspectRatioModes(state.replace(ACKs.hdmi_ack.value, b"")).name
+
     def _get_power_state(self) -> str:
         """
         Return the current power state
 
         Returns str: values of PowerStates
         """
-        success = False
-
-        cmd = (
-            Header.reference.value
-            + Header.pj_unit.value
-            + Commands.power_status.value
-            + Footer.close.value
-        )
-        # try in case we get conn refused
-        # Try to prevent power state flapping
-        msg, success = self._send_command(
-            cmd,
-            ack=ACKs.power_ack.value,
-            command_type=Header.reference.value,
-        )
-
-        # Handle error with unexpected acks
-        if not success:
-            self.logger.error("Error getting power state: %s", msg)
-            return success
-
         # remove the headers
-        state = self._replace_headers(msg)
+        state, _ = self._do_reference_op("power", ACKs.power_ack)
 
         return PowerStates(state.replace(ACKs.power_ack.value, b"")).name
 
